@@ -1,7 +1,7 @@
 """Top-level package for igel."""
-from .igel import IgelModel, models_dict, metrics_dict
+from .igel import Igel, models_dict, metrics_dict
 
 
 __author__ = "Nidhal Baccouri"
 __email__ = 'nidhalbacc@gmail.com'
-__version__ = '0.1.8'
+__version__ = '0.1.9'
